/****************************************************************************
**
** Copyright (C) 2014 Hevele Hegyi Istvan.
**
** This file is part of the fairySQL project.
**
** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
** Contact: http://www.qt-project.org/legal
**
** $QT_BEGIN_LICENSE:BSD$
** You may use this file under the terms of the BSD license as follows:
**
** "Redistribution and use in source and binary forms, with or without
** modification, are permitted provided that the following conditions are
** met:
**   * Redistributions of source code must retain the above copyright
**     notice, this list of conditions and the following disclaimer.
**   * Redistributions in binary form must reproduce the above copyright
**     notice, this list of conditions and the following disclaimer in
**     the documentation and/or other materials provided with the
**     distribution.
**   * Neither the name of Digia Plc and its Subsidiary(-ies) nor the names
**     of its contributors may be used to endorse or promote products derived
**     from this software without specific prior written permission.
**
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
** "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
** LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
** A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
** OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
** SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
** LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
** DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
** THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
** (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
** OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE."
**
** $QT_END_LICENSE$
**
****************************************************************************/

#include <QFileDialog>
#include <QSettings>
#include <QDialog>
#include <QVBoxLayout>
#include <QSqlQueryModel>
#include <QTableView>
#include <QSqlRecord>
#include <QPushButton>
#include <QSqlQuery>
#include <QMessageBox>
#include <QSqlError>
#include <QCompleter>
#include <QAction>
#include <QMenu>
#include <QContextMenuEvent>
#include <QDate>
#include <QCalendarWidget>

#include "lineedits.h"
#include "keyboard.h"
#include "global.h"
#include "calculator.h"

// QReadOnlyLineEdit

QReadOnlyLineEdit::QReadOnlyLineEdit(const QString &contents, QWidget *parent) :
    QLineEdit(contents, parent)
{
    QFont font = this->font();
    font.setPointSize(font.pointSize() + editFontPlus);
    setFont(font);
    setReadOnly(true);
}

// QFairyLineEdit

QFairyLineEdit::QFairyLineEdit(const QString &contents, QWidget *parent) :
    QLineEdit(contents, parent)
{
    QFont font = this->font();
    font.setPointSize(font.pointSize() + editFontPlus);
    setFont(font);
    QCompleter *fairyLineEditCompleter = new QCompleter(wordList, this);
    fairyLineEditCompleter->setCaseSensitivity(Qt::CaseInsensitive);
    setCompleter(fairyLineEditCompleter);
}

void QFairyLineEdit::focusOutEvent(QFocusEvent *event)
{
    if (!wordList.contains(text()))
        wordList << text();
    QLineEdit::focusOutEvent(event);
}

void QFairyLineEdit::contextMenuEvent(QContextMenuEvent *event)
{
    if (isReadOnly())
        return;
    QAction *searchAct = new QAction(tr("Keyboard"), this);
    connect(searchAct, SIGNAL(triggered()), this, SLOT(keyboard()));

    QMenu *menu = createStandardContextMenu();
    menu->addAction(searchAct);
    menu->exec(event->globalPos());
    delete menu;
}

int QFairyLineEdit::valid()
{
    return 1;
}

void QFairyLineEdit::mouseDoubleClickEvent(QMouseEvent *event)
{
    //if (!isReadOnly())
    searchItem();
}

void QFairyLineEdit::search()
{
    //if (!isReadOnly())
    searchItem();
}

void QFairyLineEdit::keyboard()
{
    //if (isReadOnly())
    //    return;
    QFairyKeyboard *fairyKeyboard = new QFairyKeyboard("fairylineedits1", text(), tr("Keyboard"), this);
    QPoint pos = this->mapToGlobal(QPoint(0, this->height() - 2));
    if (fairyKeyboard->exec())
    {
        fairyKeyboard->move(pos);
        if (fairyKeyboard->result() == QDialog::Accepted )
        {
            setText(fairyKeyboard->getValue());
        }
    }
    delete fairyKeyboard;
}

void QFairyLineEdit::searchItem()
{
    //if (!isReadOnly())
    keyboard();
}

void QFairyLineEdit::appendText(QString t)
{
    setText(text()+t);
}

void QFairyLineEdit::keyPressEvent(QKeyEvent *event)
{
    switch (event->key())
    {
    case Qt::Key_Apostrophe:
        QMessageBox::critical(this, tr("Error"),tr("Forbidden character use `"));
        break;
    case Qt::Key_Backslash:
        QMessageBox::critical(this, tr("Error"),tr("Forbidden character use /"));
        break;
    case Qt::Key_F9:
        //if (!isReadOnly())
        searchItem();
        break;
    default:
        QLineEdit::keyPressEvent(event);
    }
}

// QNoEmptyLineEdit

QNoEmptyLineEdit::QNoEmptyLineEdit(bool show_message, const QString &contents, QWidget *parent) :
    QFairyLineEdit(contents, parent)
{
    showMessage = show_message;
}

int QNoEmptyLineEdit::valid()
{
    if (text().isEmpty())
    {
        setFocus(Qt::OtherFocusReason);
        if (showMessage)
        {
            QMessageBox::critical(this, tr("Error"),tr("This field cannot be empty."));
        }
        else
        {
            setToolTip(tr("This field cannot be empty."));
            setStatusTip(tr("This field cannot be empty."));
        }
        return 0;
    }

    return 1;
}

// QDoubleLineEdit

QDoubleLineEdit::QDoubleLineEdit(const QString &contents, QWidget *parent) :
    QFairyLineEdit(contents, parent)
{
    setCompleter(0);
    QDoubleValidator *dv = new QDoubleValidator(this);
    setValidator(dv);
}

int QDoubleLineEdit::valid()
{
    return 1;
}

void QDoubleLineEdit::searchItem()
{
    QCalculatorDialog *d = new QCalculatorDialog(true, "QCalculatorDialog", tr("Calculator"), this);

    if (d->exec())
    {
        if (d->result() == QDialog::Accepted )
        {
            setText(d->getValue());
            emit textEdited(text());
        }
    }

    delete d;
}

// QNoEmptyDoubleLineEdit

QNoEmptyDoubleLineEdit::QNoEmptyDoubleLineEdit(bool show_message, const QString &contents, QWidget *parent) :
    QNoEmptyLineEdit(show_message, contents, parent)
{
    showMessage = show_message;

    setCompleter(0);
    QDoubleValidator *dv = new QDoubleValidator(this);
    setValidator(dv);
    setAlignment(Qt::AlignRight);
}

int QNoEmptyDoubleLineEdit::valid()
{
    if (text().isEmpty())
    {
        setFocus(Qt::OtherFocusReason);
        if (showMessage)
        {
            QMessageBox::critical(this, tr("Eroare 10"),tr("This field cannot be empty."));
        }
        else
        {
            setToolTip(tr("This field cannot be empty."));
            setStatusTip(tr("This field cannot be empty."));
        }
        return 0;
    }

    return 1;
}

void QNoEmptyDoubleLineEdit::searchItem()
{
    QCalculatorDialog *d = new QCalculatorDialog(true, "QCalculatorDialog", tr("Calculator"), this);

    if (d->exec())
    {
        if (d->result() == QDialog::Accepted )
        {
            setText(d->getValue());
        }
    }

    delete d;
}

// QNoZeroDoubleLineEdit

QNoZeroDoubleLineEdit::QNoZeroDoubleLineEdit(bool show_message, const QString &contents, QWidget *parent) :
    QDoubleLineEdit(contents, parent)
{
    setCompleter(0);
    QDoubleValidator *dv = new QDoubleValidator(this);
    setValidator(dv);
    showMessage = show_message;
}

int QNoZeroDoubleLineEdit::valid()
{
    double i;
    i = text().toDouble();

    if (i == 0)
    {
        if (showMessage)
            QMessageBox::critical(this, tr("Error"),tr("This field cannot be empty."));
        else
            setToolTip(tr("This field cannot be empty."));
        setFocus(Qt::OtherFocusReason);
        return 0;
    }

    return 1;
}

// QDateLineEdit

QDateLineEdit::QDateLineEdit(bool v, const QString &contents, QWidget *parent) :
    QFairyLineEdit(contents, parent)
{
    verify = v;
}

int QDateLineEdit::valid()
{
    if ((!verify) && (text().isEmpty()))
        return 1;
    QDate date;
    date = QDate::fromString(text(), Qt::ISODate);
    if (date.isValid())
        return 1;
    else
    {
        QMessageBox::critical(this, tr("Error"),tr("Enter a valid date"));
        setFocus(Qt::OtherFocusReason);
        return 0;
    }
}

void QDateLineEdit::searchItem()
{
    QDate datum;
    datum = QDate::fromString(text(), Qt::ISODate);

    QFairyModalDialog *calendarDialog = new QFairyModalDialog("fairylineedits1", tr(""), this);
    calendarDialog->setWindowTitle(tr("Data"));

    QVBoxLayout *verticalLayout = new QVBoxLayout(calendarDialog);

    QCalendarWidget *calendarWidget = new QCalendarWidget(calendarDialog);
    calendarWidget->setFirstDayOfWeek(Qt::DayOfWeek(1));
    calendarWidget->setHorizontalHeaderFormat(QCalendarWidget::SingleLetterDayNames);
    calendarWidget->setVerticalHeaderFormat(QCalendarWidget::NoVerticalHeader);
    calendarWidget->setGridVisible(true);
    calendarWidget->setSelectedDate(datum);

    verticalLayout->addWidget(calendarWidget);

    QPushButton *okButton = new QPushButton(QIcon(":images/ok.svg"),tr("OK"), this);
    QPushButton *cancelButton = new QPushButton(QIcon(":images/cancel.svg"),tr("Cancel"), this);

    connect(okButton, SIGNAL(clicked()), calendarDialog, SLOT(accept()));
    connect(cancelButton, SIGNAL(clicked()), calendarDialog, SLOT(reject()));
    connect(calendarWidget, SIGNAL(activated(QDate)), calendarDialog, SLOT(accept()));

    QHBoxLayout *buttonLayout = new QHBoxLayout;

    buttonLayout->addStretch(1);
    buttonLayout->addWidget(okButton);
    buttonLayout->addWidget(cancelButton);

    verticalLayout->addLayout(buttonLayout);

    calendarDialog->setLayout(verticalLayout);

    if (calendarDialog->exec())
    {
        setText(calendarWidget->selectedDate().toString(Qt::ISODate));
    }

    delete calendarDialog;
}

// QSavePathLineEdit

QSavePathLineEdit::QSavePathLineEdit(QString title, const QString &contents, QWidget *parent) :
    QFairyLineEdit(contents, parent)
{
    searchTitle = title;
}

void QSavePathLineEdit::searchItem()
{

    QString dir = QFileDialog::getExistingDirectory(0, searchTitle,
                                                    text(),QFileDialog::ShowDirsOnly| QFileDialog::DontResolveSymlinks);

    if (!dir.isEmpty())
        setText(dir);
}

// QSaveFileLineEdit

QSaveFileLineEdit::QSaveFileLineEdit(const QString &contents, QWidget *parent) :
    QFairyLineEdit(contents, parent)
{
}

void QSaveFileLineEdit::searchItem()
{

    QString fileName = QFileDialog::getSaveFileName();

    if (!fileName.isEmpty())
        setText(fileName);
}
